@@ -23,7 +23,6 @@
 #include <QtCore/qxmlstream.h>
 
 #include <QtCore/qcoreapplication.h>
-#include <QtCore/qscopedpointer.h>
 #include <QtCore/qdebug.h>
 #include <QtCore/qmetaobject.h>
 #include <QtCore/qset.h>
@@ -31,6 +30,8 @@
 #include <QtCore/qtextstream.h>
 #include <QtCore/qcoreapplication.h>
 
+#include <memory>
+
 QT_BEGIN_NAMESPACE
 
 using namespace Qt::StringLiterals;
@@ -559,8 +560,8 @@ static QString xmlFromWidgetBox(const QDesignerFormEditorInterface *core, const
     const bool found = QDesignerWidgetBox::findWidget(core->widgetBox(), className, QString(), &widget);
     if (!found)
         return QString();
-    QScopedPointer<DomUI> domUI(QDesignerWidgetBox::xmlToUi(className, widget.domXml(), false));
-    if (domUI.isNull())
+    std::unique_ptr<DomUI> domUI(QDesignerWidgetBox::xmlToUi(className, widget.domXml(), false));
+    if (!domUI)
         return QString();
     domUI->setAttributeVersion(u"4.0"_s);
     DomWidget *domWidget = domUI->elementWidget();
@@ -657,7 +658,7 @@ QString WidgetDataBase::formTemplate(const QDesignerFormEditorInterface *core, c
 // Set a fixed size on a XML template
 QString WidgetDataBase::scaleFormTemplate(const QString &xml, const QSize &size, bool fixed)
 {
-    QScopedPointer<DomUI> domUI(QDesignerWidgetBox::xmlToUi(u"Form"_s, xml, false));
+    std::unique_ptr<DomUI> domUI(QDesignerWidgetBox::xmlToUi(u"Form"_s, xml, false));
     if (!domUI)
         return QString();
     DomWidget *domWidget = domUI->elementWidget();
@@ -672,7 +673,7 @@ QString WidgetDataBase::scaleFormTemplate(const QString &xml, const QSize &size,
     DomProperty *maximumSizeProperty = nullptr;
 
     auto properties = domWidget->elementProperty();
-    for (DomProperty *p : properties) {
+    for (DomProperty *p : std::as_const(properties)) {
         const QString name = p->attributeName();
         if (name == geometryPropertyName) {
             geomProperty = p;
