@@ -17,6 +17,8 @@
 #include <QtWidgets/qframe.h>
 #include <QtWidgets/qabstractscrollarea.h>
 
+#include <limits>
+
 QT_BEGIN_NAMESPACE
 
 using namespace Qt::StringLiterals;
@@ -26,14 +28,6 @@ namespace QFormInternal
 {
 #endif
 
-static inline void fixEnum(QString &s)
-{
-    qsizetype qualifierIndex = s.lastIndexOf(u':');
-    if (qualifierIndex == -1)
-        qualifierIndex = s.lastIndexOf(u'.');
-    if (qualifierIndex != -1)
-        s.remove(0, qualifierIndex + 1);
-}
 // Convert complex DOM types with the help of  QAbstractFormBuilder
 QVariant domPropertyToVariant(QAbstractFormBuilder *afb,const QMetaObject *meta,const  DomProperty *p)
 {
@@ -73,29 +67,44 @@ QVariant domPropertyToVariant(QAbstractFormBuilder *afb,const QMetaObject *meta,
 
         const QMetaEnum e = meta->property(index).enumerator();
         Q_ASSERT(e.isFlag() == true);
-        return QVariant(e.keysToValue(p->elementSet().toUtf8()));
+        bool ok{};
+        QVariant result(e.keysToValue(p->elementSet().toUtf8().constData(), &ok));
+        if (!ok) {
+            uiLibWarning(QCoreApplication::translate("QFormBuilder",
+                                                     "The value \"%1\" of the set-type property %2 could not be read.").
+                                                     arg(p->elementSet(), p->attributeName()));
+            return {};
+        }
+        return result;
     }
 
     case DomProperty::Enum: {
         const QByteArray pname = p->attributeName().toUtf8();
         const int index = meta->indexOfProperty(pname);
-        QString enumValue = p->elementEnum();
+        const auto &enumValue = p->elementEnum();
         // Triggers in case of objects in Designer like Spacer/Line for which properties
         // are serialized using language introspection. On preview, however, these objects are
         // emulated by hacks in the formbuilder (size policy/orientation)
-        fixEnum(enumValue);
         if (index == -1) {
             // ### special-casing for Line (QFrame) -- fix for 4.2. Jambi hack for enumerations
             if (!qstrcmp(meta->className(), "QFrame")
                 && (pname == QByteArray("orientation"))) {
-                return QVariant(enumValue == QFormBuilderStrings::instance().horizontalPostFix ? QFrame::HLine : QFrame::VLine);
+                return QVariant(enumValue.endsWith("Horizontal"_L1) ? QFrame::HLine : QFrame::VLine);
             }
             uiLibWarning(QCoreApplication::translate("QFormBuilder", "The enumeration-type property %1 could not be read.").arg(p->attributeName()));
             return QVariant();
         }
 
         const QMetaEnum e = meta->property(index).enumerator();
-        return QVariant(e.keyToValue(enumValue.toUtf8()));
+        bool ok{};
+        QVariant result(e.keyToValue(enumValue.toUtf8().constData(), &ok));
+        if (!ok) {
+            uiLibWarning(QCoreApplication::translate("QFormBuilder",
+                                                     "The value \"%1\" of the enum-type property %2 could not be read.").
+                         arg(enumValue, p->attributeName()));
+            return {};
+        }
+        return result;
     }
     case DomProperty::Brush:
         return QVariant::fromValue(afb->setupBrush(p->elementBrush()));
@@ -111,13 +120,22 @@ QVariant domPropertyToVariant(QAbstractFormBuilder *afb,const QMetaObject *meta,
     return domPropertyToVariant(p);
 }
 
+// Convert a legacy Qt 4 integer font weight to the closes enumeration value
+
+static inline QMetaEnum fontWeightMetaEnum()
+{
+    const QMetaEnum result = metaEnum<QAbstractFormBuilderGadget>("fontWeight");
+    Q_ASSERT(result.isValid());
+    return result;
+}
+
 // Convert simple DOM types
 QVariant domPropertyToVariant(const DomProperty *p)
 {
     // requires non-const virtual nameToIcon, etc.
     switch(p->kind()) {
     case DomProperty::Bool:
-        return QVariant(p->elementBool() == QFormBuilderStrings::instance().trueValue);
+        return QVariant(p->elementBool() == "true"_L1);
 
     case DomProperty::Cstring:
         return QVariant(p->elementCstring().toUtf8());
@@ -196,8 +214,6 @@ QVariant domPropertyToVariant(const DomProperty *p)
             f.setPointSize(font->elementPointSize());
         if (font->hasElementItalic())
             f.setItalic(font->elementItalic());
-        if (font->hasElementBold())
-            f.setBold(font->elementBold());
         if (font->hasElementUnderline())
             f.setUnderline(font->elementUnderline());
         if (font->hasElementStrikeOut())
@@ -214,6 +230,15 @@ QVariant domPropertyToVariant(const DomProperty *p)
             f.setHintingPreference(enumKeyOfObjectToValue<QAbstractFormBuilderGadget, QFont::HintingPreference>("hintingPreference",
                                                                                                                 font->elementHintingPreference().toLatin1().constData()));
         }
+
+        if (font->hasElementFontWeight()) {
+            f.setWeight(enumKeyOfObjectToValue<QAbstractFormBuilderGadget, QFont::Weight>(
+                "fontWeight",
+                font->elementFontWeight().toLatin1().constData()));
+        } else if (font->hasElementBold()) {
+            f.setBold(font->elementBold());
+        }
+
         return QVariant::fromValue(f);
     }
 
@@ -267,14 +292,14 @@ QVariant domPropertyToVariant(const DomProperty *p)
         if (sizep->hasElementHSizeType()) {
             sizePolicy.setHorizontalPolicy((QSizePolicy::Policy) sizep->elementHSizeType());
         } else if (sizep->hasAttributeHSizeType()) {
-            const QSizePolicy::Policy sp = enumKeyToValue<QSizePolicy::Policy>(sizeType_enum, sizep->attributeHSizeType().toLatin1());
+            const auto sp = enumKeyToValue<QSizePolicy::Policy>(sizeType_enum, sizep->attributeHSizeType().toLatin1());
             sizePolicy.setHorizontalPolicy(sp);
         }
 
         if (sizep->hasElementVSizeType()) {
             sizePolicy.setVerticalPolicy((QSizePolicy::Policy) sizep->elementVSizeType());
         } else if (sizep->hasAttributeVSizeType()) {
-            const  QSizePolicy::Policy sp = enumKeyToValue<QSizePolicy::Policy>(sizeType_enum, sizep->attributeVSizeType().toLatin1());
+            const  auto sp = enumKeyToValue<QSizePolicy::Policy>(sizeType_enum, sizep->attributeVSizeType().toLatin1());
             sizePolicy.setVerticalPolicy(sp);
         }
 
@@ -297,7 +322,7 @@ static bool applySimpleProperty(const QVariant &v, bool translateString, DomProp
 {
     switch (v.metaType().id()) {
     case QMetaType::QString: {
-        DomString *str = new DomString();
+        auto *str = new DomString();
         str->setText(v.toString());
         if (!translateString)
             str->setAttributeNotr(u"true"_s);
@@ -330,11 +355,11 @@ static bool applySimpleProperty(const QVariant &v, bool translateString, DomProp
         return true;
 
     case QMetaType::Bool:
-        dom_prop->setElementBool(v.toBool() ? QFormBuilderStrings::instance().trueValue : QFormBuilderStrings::instance().falseValue);
+        dom_prop->setElementBool(v.toBool() ? "true"_L1 : "false"_L1);
         return true;
 
     case QMetaType::QChar: {
-        DomChar *ch = new DomChar();
+        auto *ch = new DomChar();
         const QChar character = v.toChar();
         ch->setElementUnicode(character.unicode());
         dom_prop->setElementChar(ch);
@@ -342,7 +367,7 @@ static bool applySimpleProperty(const QVariant &v, bool translateString, DomProp
         return true;
 
     case QMetaType::QPoint: {
-        DomPoint *pt = new DomPoint();
+        auto *pt = new DomPoint();
         const QPoint point = v.toPoint();
         pt->setElementX(point.x());
         pt->setElementY(point.y());
@@ -351,7 +376,7 @@ static bool applySimpleProperty(const QVariant &v, bool translateString, DomProp
         return true;
 
     case QMetaType::QPointF: {
-        DomPointF *ptf = new DomPointF();
+        auto *ptf = new DomPointF();
         const QPointF pointf = v.toPointF();
         ptf->setElementX(pointf.x());
         ptf->setElementY(pointf.y());
@@ -360,8 +385,8 @@ static bool applySimpleProperty(const QVariant &v, bool translateString, DomProp
         return true;
 
     case QMetaType::QColor: {
-        DomColor *clr = new DomColor();
-        const QColor color = qvariant_cast<QColor>(v);
+        auto *clr = new DomColor();
+        const auto color = qvariant_cast<QColor>(v);
         clr->setElementRed(color.red());
         clr->setElementGreen(color.green());
         clr->setElementBlue(color.blue());
@@ -373,7 +398,7 @@ static bool applySimpleProperty(const QVariant &v, bool translateString, DomProp
         return true;
 
     case QMetaType::QSize: {
-        DomSize *sz = new DomSize();
+        auto *sz = new DomSize();
         const QSize size = v.toSize();
         sz->setElementWidth(size.width());
         sz->setElementHeight(size.height());
@@ -382,7 +407,7 @@ static bool applySimpleProperty(const QVariant &v, bool translateString, DomProp
         return true;
 
     case QMetaType::QSizeF: {
-        DomSizeF *szf = new DomSizeF();
+        auto *szf = new DomSizeF();
         const QSizeF sizef = v.toSizeF();
         szf->setElementWidth(sizef.width());
         szf->setElementHeight(sizef.height());
@@ -391,7 +416,7 @@ static bool applySimpleProperty(const QVariant &v, bool translateString, DomProp
         return true;
 
     case QMetaType::QRect: {
-        DomRect *rc = new DomRect();
+        auto *rc = new DomRect();
         const QRect rect = v.toRect();
         rc->setElementX(rect.x());
         rc->setElementY(rect.y());
@@ -402,7 +427,7 @@ static bool applySimpleProperty(const QVariant &v, bool translateString, DomProp
         return true;
 
     case QMetaType::QRectF: {
-        DomRectF *rcf = new DomRectF();
+        auto *rcf = new DomRectF();
         const QRectF rectf = v.toRectF();
         rcf->setElementX(rectf.x());
         rcf->setElementY(rectf.y());
@@ -413,11 +438,25 @@ static bool applySimpleProperty(const QVariant &v, bool translateString, DomProp
         return true;
 
     case QMetaType::QFont: {
-        DomFont *fnt = new DomFont();
-        const QFont font = qvariant_cast<QFont>(v);
+        auto *fnt = new DomFont();
+        const auto font = qvariant_cast<QFont>(v);
         const uint mask = font.resolveMask();
-        if (mask & QFont::WeightResolved)
-            fnt->setElementBold(font.bold());
+        if (mask & QFont::WeightResolved) {
+            switch (font.weight()) {
+            case QFont::Normal:
+                fnt->setElementBold(false);
+                break;
+            case QFont::Bold:
+                fnt->setElementBold(true);
+                break;
+            default: {
+                const QMetaEnum me = fontWeightMetaEnum();
+                const QString ws = QLatin1StringView(me.valueToKey(font.weight()));
+                fnt->setElementFontWeight(ws);
+            }
+                break;
+            }
+        }
         if ((mask & (QFont::FamilyResolved | QFont::FamiliesResolved)) != 0)
             fnt->setElementFamily(font.family());
         if (mask & QFont::StyleResolved)
@@ -452,15 +491,15 @@ static bool applySimpleProperty(const QVariant &v, bool translateString, DomProp
 #endif
 
     case QMetaType::QKeySequence: {
-        DomString *s = new DomString();
+        auto *s = new DomString();
         s->setText(qvariant_cast<QKeySequence>(v).toString(QKeySequence::PortableText));
         dom_prop->setElementString(s);
         }
         return true;
 
     case QMetaType::QLocale: {
-        DomLocale *dom = new DomLocale();
-        const QLocale locale = qvariant_cast<QLocale>(v);
+        auto *dom = new DomLocale();
+        const auto locale = qvariant_cast<QLocale>(v);
 
         const QMetaEnum language_enum = metaEnum<QAbstractFormBuilderGadget>("language");
         const QMetaEnum territory_enum = metaEnum<QAbstractFormBuilderGadget>("country");
@@ -473,8 +512,8 @@ static bool applySimpleProperty(const QVariant &v, bool translateString, DomProp
         return true;
 
     case QMetaType::QSizePolicy: {
-        DomSizePolicy *dom = new DomSizePolicy();
-        const QSizePolicy sizePolicy = qvariant_cast<QSizePolicy>(v);
+        auto *dom = new DomSizePolicy();
+        const auto sizePolicy = qvariant_cast<QSizePolicy>(v);
 
         dom->setElementHorStretch(sizePolicy.horizontalStretch());
         dom->setElementVerStretch(sizePolicy.verticalStretch());
@@ -489,8 +528,8 @@ static bool applySimpleProperty(const QVariant &v, bool translateString, DomProp
         return true;
 
     case QMetaType::QDate: {
-        DomDate *dom = new DomDate();
-        const QDate date = qvariant_cast<QDate>(v);
+        auto *dom = new DomDate();
+        const auto date = qvariant_cast<QDate>(v);
 
         dom->setElementYear(date.year());
         dom->setElementMonth(date.month());
@@ -501,8 +540,8 @@ static bool applySimpleProperty(const QVariant &v, bool translateString, DomProp
         return true;
 
     case QMetaType::QTime: {
-        DomTime *dom = new DomTime();
-        const QTime time = qvariant_cast<QTime>(v);
+        auto *dom = new DomTime();
+        const auto time = qvariant_cast<QTime>(v);
 
         dom->setElementHour(time.hour());
         dom->setElementMinute(time.minute());
@@ -513,8 +552,8 @@ static bool applySimpleProperty(const QVariant &v, bool translateString, DomProp
         return true;
 
     case QMetaType::QDateTime: {
-        DomDateTime *dom = new DomDateTime();
-        const QDateTime dateTime = qvariant_cast<QDateTime>(v);
+        auto *dom = new DomDateTime();
+        const auto dateTime = qvariant_cast<QDateTime>(v);
 
         dom->setElementHour(dateTime.time().hour());
         dom->setElementMinute(dateTime.time().minute());
@@ -528,10 +567,10 @@ static bool applySimpleProperty(const QVariant &v, bool translateString, DomProp
         return true;
 
     case QMetaType::QUrl: {
-        DomUrl *dom = new DomUrl();
+        auto *dom = new DomUrl();
         const QUrl url = v.toUrl();
 
-        DomString *str = new DomString();
+        auto *str = new DomString();
         str->setText(url.toString());
         dom->setElementString(str);
 
@@ -540,7 +579,7 @@ static bool applySimpleProperty(const QVariant &v, bool translateString, DomProp
         return true;
 
     case QMetaType::QStringList: {
-        DomStringList *sl = new DomStringList;
+        auto *sl = new DomStringList;
         sl->setElementString(qvariant_cast<QStringList>(v));
         dom_prop->setElementStringList(sl);
     }
@@ -570,10 +609,9 @@ static bool isOfType(const QMetaObject *what, const QMetaObject *type)
 
 static bool isTranslatable(const QString &pname, const QVariant &v, const QMetaObject *meta)
 {
-    const QFormBuilderStrings &strings = QFormBuilderStrings::instance();
-    if (pname == strings.objectNameProperty)
+    if (pname == "objectName"_L1)
         return false;
-    if (pname == strings.styleSheetProperty && v.metaType().id() == QMetaType::QString
+    if (pname == "styleSheet"_L1 && v.metaType().id() == QMetaType::QString
         && isOfType(meta, &QWidget::staticMetaObject)) {
         return false;
     }
@@ -585,15 +623,13 @@ static bool isTranslatable(const QString &pname, const QVariant &v, const QMetaO
 DomProperty *variantToDomProperty(QAbstractFormBuilder *afb, const QMetaObject *meta,
                                   const QString &pname, const QVariant &v)
 {
-    const QFormBuilderStrings &strings = QFormBuilderStrings::instance();
-
-    DomProperty *dom_prop = new DomProperty();
+    auto *dom_prop = new DomProperty();
     dom_prop->setAttributeName(pname);
 
     const int pindex = meta->indexOfProperty(pname.toLatin1());
     if (pindex != -1) {
         QMetaProperty meta_property = meta->property(pindex);
-        if ((v.metaType().id() == QMetaType::Int || v.metaType().id() == QMetaType::UInt) && meta_property.isEnumType()) {
+        if (v.canConvert<int>() && meta_property.isEnumType()) {
             const QMetaEnum e = meta_property.enumerator();
             if (e.isFlag())
                 dom_prop->setElementSet(QString::fromLatin1(e.valueToKeys(v.toInt())));
@@ -601,8 +637,11 @@ DomProperty *variantToDomProperty(QAbstractFormBuilder *afb, const QMetaObject *
                 dom_prop->setElementEnum(QString::fromLatin1(e.valueToKey(v.toInt())));
             return dom_prop;
         }
-        if (!meta_property.hasStdCppSet() || (isOfType(meta, &QAbstractScrollArea::staticMetaObject) && pname == strings.cursorProperty))
+        if (!meta_property.hasStdCppSet()
+            || (isOfType(meta, &QAbstractScrollArea::staticMetaObject)
+                && pname == "cursor"_L1)) {
             dom_prop->setAttributeStdset(0);
+        }
     }
 
     // Try simple properties
@@ -612,8 +651,8 @@ DomProperty *variantToDomProperty(QAbstractFormBuilder *afb, const QMetaObject *
     // Complex properties
     switch (v.metaType().id()) {
     case QMetaType::QPalette: {
-        DomPalette *dom = new DomPalette();
-        QPalette palette = qvariant_cast<QPalette>(v);
+        auto *dom = new DomPalette();
+        auto palette = qvariant_cast<QPalette>(v);
 
         palette.setCurrentColorGroup(QPalette::Active);
         dom->setElementActive(afb->saveColorGroup(palette));
